BUILD_BACKEND = "hatchling.build"
DEFAULT_BUILD_DIRECTORY = "dist"
DEFAULT_BUILD_SCRIPT = "hatch_build.py"
DEFAULT_CONFIG_FILE = "hatch.toml"


class BuildEnvVars:
    REQUESTED_TARGETS = "HATCH_BUILD_REQUESTED_TARGETS"
    LOCATION = "HATCH_BUILD_LOCATION"
    HOOKS_ONLY = "HATCH_BUILD_HOOKS_ONLY"
    NO_HOOKS = "HATCH_BUILD_NO_HOOKS"
    HOOKS_ENABLE = "HATCH_BUILD_HOOKS_ENABLE"
    HOOK_ENABLE_PREFIX = "HATCH_BUILD_HOOK_ENABLE_"
    CLEAN = "HATCH_BUILD_CLEAN"
    CLEAN_HOOKS_AFTER = "HATCH_BUILD_CLEAN_HOOKS_AFTER"
